'''
Handles parsing of the .supervisorconf file.
'''

import os

class InvalidEventValue(Exception):
	pass

class InvalidPathValue(Exception):
	pass

class Parser:
	def __init__(self):
		self.config = None
	
	def _verify_event(self, event):
		# Make sure the event values are valid
		events = ['any', 'created', 'modified', 'deleted', 'moved']
		if event not in events:
			raise InvalidEventValue(f'\n\nYou cannot use "{event}" as an event value.\n\nThe available options are: {events}\n')
		
	def _verify_path(self, path):
		# Make sure the requested path exists
		if not os.path.exists(path):
			raise InvalidPathValue(f'The path, "{path}" does not exist.')

	def _verify_values(self, lines):
		# Handles running error-checking parse
		# functions on different parts of a config
		# line.
		for line in lines:
			self._verify_event(line[0])
			self._verify_path(line[1])


	def parse(self, config):
		# @returns like: [['any', '/path/', 'command']]
		lines = [line.split(":::") for line in config]

		self._verify_values(lines)
		return lines


import sys
import time

from .config import Config
from .parser import Parser
from .logger import Logger
from .watcher import Watcher

class Supervisor:
	def __init__(self, settings):
		self.settings = settings

		self.config = Config()
		self.parser = Parser()
		self.logger = Logger()
		self.watcher = Watcher(self.logger)
	
	def _handle_special_arguments(self):
		if self.settings.get('log'):
			self.logger.should_output = True
			self.logger.log_to_file('==========')
				
		if self.settings.get('pass_argument'):
			self.watcher.pass_argument()

	def __call__(self):
		self._handle_special_arguments()
		self.watcher(self.parser.parse(self.config._get_config()), self.settings)


import click

from lib.main import Supervisor
from lib.config import Config

# To keep things simple, each option here simply
# modifies a particular configuration value
# which the Supervisor class accepts as an 
# argument.
settings = {}

config = Config()

conf_options = {
	'create': config.create,
	'reset': config.reset,
	'edit': config.edit,
	'delete': config.delete,
	'echo': config.echo
}

def set_config(name, value):
	settings.update({ name: value })

def modify_and_inform(name, value, msg):
	set_config(name, value)
	
	if value:
		click.echo(msg)

##########

@click.command()
@click.option('-c', '--conf', default=None)
@click.option('-r', '--recursive', is_flag=True, default=False)
@click.option('-l', '--log', is_flag=True, default=False)
@click.option('-p', '--pass-argument', is_flag=True, default=False)
def cli(conf, recursive, log, pass_argument):
	modify_and_inform('log', log, 'Logging enabled.')
	modify_and_inform('recursive', recursive, 'Recursive walking enabled.')
	modify_and_inform('pass_argument', pass_argument, 'Passing changed file as an argument.')

	if conf:
		try:
			conf_options[conf]()
		except KeyError:
			print(f'"{conf}" is not a configuration command.')
		quit()
	else:
		supervisor = Supervisor(settings)
		supervisor()

##########

if __name__ == '__main__':
	cli()


import os
from subprocess import call
from platform import system
from pathlib import Path

from .process import Process

class Config:
	def __init__(self):
		self.config_file = os.path.join(Path.home(), '.supervisor')
		self.process = Process()

	## Creation and Deletion
	def _create_config_file(self):
		try:
			with open(self.config_file, 'x') as supervisor_conf:
				supervisor_conf.write('# Read the Supervisor README')
				
			print(f'Created: {self.config_file}. Use "supervisor -c edit" to modify it.')
		except FileExistsError: # you tell me
			pass

	def _check_config(self):
		if os.path.exists(self.config_file):
			print('You already have a ".supervisor" file in your home folder.')
			quit()
		
		self._create_config_file()

	def _reset_configuration_file(self):
		os.remove(self.config_file)
		
		if input("Create a new configuration file? (y/n) ") == "y":
			self.create()
		else:
			quit()

	def _confirm_reset(self):
		if input('Are you sure? This will delete .supervisor (y/n) ') == 'y':
			self._reset_configuration_file()
		else:
			quit()


	## Handlers
	def _get_config(self):
		try:
			with open(self.config_file, 'r') as conf_file:
				raw_lines = conf_file.readlines()
				lines = [line.strip() for line in raw_lines]

				# Returns all lines that aren't empty and don't
				# start with "#"
				return [line for line in lines if line != "" and line[0] != "#"]
		except FileNotFoundError:
			print('Configuration missing. Please run "supervisor -c create"')
			quit()
	
	def _not_found(self):
		print('.supervisor was not found in your home folder.')

		if input("Create it? (y/n) ") == "y":
			self.create()

	
	def create(self):
		self._check_config()
		self._create_config_file()

	def reset(self):
		if os.path.exists(self.config_file):
			self._confirm_reset()
		else:
			self._not_found()
	
	def edit(self):
		if os.path.exists(self.config_file):
			opener = input('Editor (press Enter for "vim"): ')

			if opener.strip() == "":
				opener = "vim"

			call([opener, self.config_file])
		else:
			self._not_found()

	def delete(self):
		try:
			self._confirm_reset()
		except FileNotFoundError:
			self._not_found()

	def echo(self):
		if os.path.exists(self.config_file):
			with open(self.config_file, 'r') as conf_file:
				print(conf_file.read())
		else:
			self._not_found()


import sys
import time
import logging
from pathlib import Path
from watchdog.observers import Observer
from watchdog.events import LoggingEventHandler

from .process import Process

class EventHandler:
	'''
	@about:
		A specific class used by Watchdog which implements explicit
		event handler functions for certain actions.
	@info:
		https://pythonhosted.org/watchdog/api.html#watchdog.events.FileSystemEventHandler
	'''
	def __init__(self, logger, config, pass_arg):
		self.logger = logger
		self.process = Process(self.logger)
		self.config = config
		self.pass_arg = pass_arg
		self.map = {
			'created': self.on_created,
			'deleted': self.on_deleted,
			'modified': self.on_modified,
			'moved': self.on_moved,
		}
	
	def _get_argument(self, event):
		# If invoked with -p or --pass, this will
		# return a string containing:
		# [script/command to run] [path of changed file]
		# If not invoked with -p, this will return only
		# the script/command to run.
		if self.pass_arg:
			return f'{self.config[2]} {event.src_path}'
		return self.config[2]

	def _run_process(self, event, event_type):
		# Performs the script/command and logs it.
		self.logger.log(f'{event_type}: [bold magenta]{event.src_path}[/bold magenta]')
		self.process(self._get_argument(event))

	def dispatch(self, event):
		# Special method for determining which event
		# to notice. This will take the first part
		# of the config file line (modified/any/deleted/etc.)
		self.on_any_event(event)
		
		if event.event_type == self.config[0]:
			self.map[event.event_type](event)

	def on_any_event(self, event):
		# If this check wasn't performed, there would be
		# no use in specifying which type of event to watch.
		# This method is run every time there is a change in
		# the folder.
		if self.config[0] == "any":
			self._run_process(event, event.event_type.capitalize())
	
	# The following 4 methods are specific
	# to Watchdog
	def on_created(self, event):
		self._run_process(event, 'Created')
	
	def on_deleted(self, event):
		self._run_process(event, 'Deleted')
	
	def on_modified(self, event):
		self._run_process(event, 'Modified')
	
	def on_moved(self, event):
		self._run_process(event, 'Moved')

class Watcher:
	def __init__(self, logger):
		self.config = None
		self.logger = logger
		self.pass_arg = False

		self.recursive = None
		self.log = None

	def pass_argument(self):
		# This method is called if Supervisor is
		# invoked with -p or --pass
		self.pass_arg = True

	def _alert(self, config):
		# Explicitly informs the user about the current
		# Watcher configuration.
		self.logger.log(f'\nEvent type: [bold magenta]{config[0]}[/bold magenta]')
		self.logger.log(f'Directory: [bold cyan]{config[1]}[/bold cyan]')
		self.logger.log(f'Operation: [bold green]{config[2]}[/bold green]\n')
		self.logger.log('----------\n')

	def _check_config(self):
		for config in self.config:
			path = config[1][:-1] if config[1].endswith('/') else config[1]
			if path == str(Path.home()) and self.log:
				print('\n'.join([
					'Watching your home directory with logging enabled is not supported.',
					'This would create an infinite loop.',
					'Either disable logging or stop watching your home directory.'
				]))
				quit()

	def _watch(self, configs):
		self._check_config()

		# The main process. The config is simply the middle
		# section of a config file line.
		observers = []
		for config in configs:
			self._alert(config)

			observer = Observer()
			observer.schedule(EventHandler(self.logger, config, self.pass_arg), config[1], recursive=self.recursive)
			observer.start()
			observers.append(observer)
		try:
			while True:
				time.sleep(1) # this will cause Ctrl+C to have a delay
		except KeyboardInterrupt:
			# Instead of this exception, we could watch stdin
			# and then begin another watcher process, if applicable.
			for observer in observers:
				observer.stop()
				observer.join()

	def __call__(self, parsed_config, settings):
		self.config = parsed_config
		self.recursive = settings.get('recursive')
		self.log = settings.get('log')
		self._watch(self.config)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ssupervisor', 'ssupervisor.lib']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0', 'rich>=10.14.0,<11.0.0', 'watchdog>=2.1.6,<3.0.0']

setup_kwargs = {
    'name': 'ssupervisor',
    'version': '1.0.1',
    'description': '',
    'long_description': None,
    'author': 'Infinium',
    'author_email': 'infinium-llc@protonamil.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import subprocess

class Process:
	def __init__(self, logger = None):
		self.logger = logger
		self.process = None

	def _run(self):
		subprocess.run(self.process.split())
		self.logger.log(f'Executed: [bold green]{self.process}[/bold green]')
	
	def __call__(self, process):
		self.process = process
		self._run()


from pathlib import Path
from rich.console import Console

class Logger:
	def __init__(self):
		self.output_file = f'{Path.home()}/supervisor.log'
		self.should_output = False
		self.console = Console()
	
	def log_to_file(self, message):
		if self.should_output:
			with open(self.output_file, 'a') as out_f:
				out_f.write(f'\n{message}')

	def log(self, message):
		self.console.print(message)
		self.log_to_file(message)